/* whole_report.c -- processes turtle stamp list without splitting it */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#define BUFSIZE       1024
#define INDEX_FILE   "index.tmp"
#define MAIN_FILE    "main.tmp"
#define OUTPUT_FILE  "all.html"
#define TITLE0       "Additions and corrections for "
#define TITLE1       "A World-Wide List of Turtles and Tortoises on Stamps"
#define TITLE2       "by Donald N. Riemer"
#define HOME         "http://www.personal.psu.edu/crr2/turtstmp"

void  htmlhead (FILE *fp, char *title);
int   getline  (char *buf);
void  weedout  (char *buf);
void  compact  (char *buf);
int   starts   (char *s, char *pat);
char *strtrim  (char *s);
char *usage    (void);
void  fatal    (char *msg);

int main (int argc, char **argv) {
	char *dname = "";      /* pacify lint */
	char *update = "";     /* pacify lint */
	char buf   [BUFSIZE];
	char iname [BUFSIZE];
	char fname [BUFSIZE];
	char oname [BUFSIZE];
	FILE *ix;
	FILE *fp = NULL;
	char ccode [BUFSIZE];
	char cname [BUFSIZE];
	char *p;
	int  letter = 'A' - 1;
	char title    [BUFSIZE];
	char dateline [BUFSIZE];
	char cmdbuf   [BUFSIZE];

	/* get arguments */
	if (argc == 2) {
		dname = argv[1];
		update = "";
	}
	else if (argc == 3) {
		dname = argv[1];
		update = argv[2];
	}
	else
		fatal (usage());

	/* check dirname */
	sprintf (cmdbuf, "test -d %s", dname);
	if (system (cmdbuf)) {     /* zero => true; non-zero => false */
		sprintf (cmdbuf, "no such directory: %s\n%s", dname, usage());
		fatal (cmdbuf);
	}

	/* start index file */
	sprintf (iname, "%s/%s", dname, INDEX_FILE);
//	fprintf (stderr, "opening %s\n", iname);
	if (!(ix = fopen (iname, "w")))
		fatal ("error opening temporary index file");
	if (*update)
		sprintf (title, "%s (%s update)", TITLE1, update);
	else
		sprintf (title, "%s (full list)", TITLE1);
	htmlhead (ix, title);
	if (*update) {
		fprintf (ix, "<center><font size=\"+1\"><b><i>%s%s</i></b></font></center>\n",
					TITLE0, update);
		fprintf (ix, "<hr width=\"50%%\">\n");
	}
	fprintf (ix, "<br>\n");
	fprintf (ix, "<center><h1>%s</h1></center>\n", TITLE1);
	fprintf (ix, "<center><h2>%s</h2></center>\n", TITLE2);
	fprintf (ix, "<br>\n");
	fprintf (ix, "<center><i>\n");
	fprintf (ix, "  <a href=\"#A\">A</a> <a href=\"#B\">B</a> <a href=\"#C\">C</a>\n");
	fprintf (ix, "  <a href=\"#D\">D</a> <a href=\"#E\">E</a> <a href=\"#F\">F</a>\n");
	fprintf (ix, "  <a href=\"#G\">G</a> <a href=\"#H\">H</a> <a href=\"#I\">I</a>\n");
	fprintf (ix, "  <a href=\"#J\">J</a> <a href=\"#K\">K</a> <a href=\"#L\">L</a>\n");
	fprintf (ix, "  <a href=\"#M\">M</a> <a href=\"#N\">N</a> <a href=\"#O\">O</a>\n");
	fprintf (ix, "  <a href=\"#P\">P</a> <a href=\"#Q\">Q</a> <a href=\"#R\">R</a>\n");
	fprintf (ix, "  <a href=\"#S\">S</a> <a href=\"#T\">T</a> <a href=\"#U\">U</a>\n");
	fprintf (ix, "  <a href=\"#V\">V</a> <a href=\"#W\">W</a> <a href=\"#X\">X</a>\n");
	fprintf (ix, "  <a href=\"#Y\">Y</a> <a href=\"#Z\">Z</a>\n");
	fprintf (ix, "</i></center>\n");
	fprintf (ix, "<p>\n");
	fprintf (ix, "<dl>\n");

	/* start main file */
	sprintf (fname, "%s/%s", dname, MAIN_FILE);
//	fprintf (stderr, "opening %s\n", fname);
	if (!(fp = fopen (fname, "w")))
		fatal ("error opening temporary main file");

	/* process input file */
	while (getchar() != '@')           /* skip preliminary printer codes */
		;
	ungetc ('@', stdin);
	while (getline (buf) != EOF) {     /* read a cleaned-up line into buf
	                                      (\n et al. replaced with \0) */
		if (starts (buf, "@@@")) {
			/* get country code and name */
			strcpy (ccode, strtrim (buf + 3));
			if ((p = strstr (ccode, ".html")))
				*p = '\0';
			if (getline (buf) == EOF)
				fatal ("unexpected EOF");
			strcpy (cname, strtrim (buf));

			/* print anchor in main file */
			fprintf (fp, "<a name=\"%s\"></a>\n", ccode);

			/* add link to index */
			while (letter < *cname) {
				++letter;
				fprintf (ix, "<dt><br><a name=\"%c\"><big>-- <b>%c</b> --</big></a>\n",
				             letter, letter);
				fprintf (ix, "<dd>\n");
			}
			fprintf (ix, "\t<a href=\"#%s\">", ccode);
			fprintf (ix, "%s</a><br>\n", cname);
		}
		else if (starts (buf, "<html>") || starts (buf, "<head>") ||
		         starts (buf, "<body ") || starts (buf, "</body>") ||
		         starts (buf, "</html>")) {
			continue;     /* skip this line */
		}
		else if (starts (buf, "<i>Last updated:")) {
			strcpy (dateline, buf);     /* save to print at end of file */
		}
		else {
			fprintf (fp, "%s\n", buf);     /* copy line as is */
		}
	}

	/* wrap up index file */
	while (letter < 'Z') {
		++letter;
		fprintf (ix, "<dt><br><a name=\"%c\"><big>-- <b>%c</b> --</big></a>\n",
		             letter, letter);
		fprintf (ix, "<dd>\n");
	}
	fprintf (ix, "</dl>\n");
	fprintf (ix, "<p>\n");
	fprintf (ix, "<hr size=5>\n");
	fprintf (ix, "<p>\n");
	fclose (ix);

	/* wrap up main file */
	fprintf (fp, "<p>\n");
	fprintf (fp, "<br>\n");
	fprintf (fp, "<table width=\"100%%\">\n");
	fprintf (fp, "<tr valign=baseline>\n");
	fprintf (fp, "<td align=left><a href=\"%s/\">%s/</a></td>\n", HOME, HOME);
	fprintf (fp, "<td align=right>%s</td>\n", dateline);
	fprintf (fp, "</tr>\n");
	fprintf (fp, "</table>\n");
	fprintf (fp, "<p>\n");
	fprintf (fp, "</body>\n");
	fprintf (fp, "</html>\n");
	fclose (fp);

	/* assemble files */
	sprintf (oname, "%s/%s", dname, OUTPUT_FILE);
//	fprintf (stderr, "catenating temp files to %s\n", oname);
	sprintf (cmdbuf, "cat %s %s > %s", iname, fname, oname);
	if (system (cmdbuf)) {     /* non-zero indicates error */
		sprintf (cmdbuf, "could not catenate %s + %s to %s", iname, fname, oname);
		fatal (cmdbuf);
	}
//	fprintf (stderr, "deleting temp files\n");
	if (unlink (iname) || unlink (fname)) {     /* non-zero indicates error */
		sprintf (cmdbuf, "could not delete %s and/or %s", iname, fname);
		fatal (cmdbuf);
	}
	exit (0);
}

void htmlhead (FILE *fp, char *title) {
	fprintf (fp, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n");
	fprintf (fp, "\t\"http://www.w3.org/TR/html4/loose.dtd\">\n");
	fprintf (fp, "<html>\n");
	fprintf (fp, "<head>\n");
	fprintf (fp, "<title>%s</title>\n", title);
	fprintf (fp, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
	fprintf (fp, "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n");
	fprintf (fp, "<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/turtstmp.css\">\n", HOME);
	fprintf (fp, "<link rel=\"shortcut icon\" href=\"%s/turtstmp.ico\">\n", HOME);
	fprintf (fp, "</head>\n");
	fprintf (fp, "<body>\n");
}

int getline (char *buf) {
	char *b;
	int c;

	if (feof (stdin))
		return EOF;
	else {
		b = buf;
		while ( (c = getchar()) != '\n' && c != 12 && c != EOF ) {
			*b++ = c;
		}
		*b = '\0';
		weedout (buf);
		compact (buf);
		return (strlen (buf));
	}
}

void weedout (char *buf) {
	char *b;
	char *p;

	for (p = b = buf; *p; p++) {
		if (*p == 13) {          /* skip ^M */
			continue;
		}
		else if (*p == 27) {     /* escape character */
			if (*++p == 'E')
				continue;      /* skip \027E */
			else if (*p == '&' && *++p == 'a' && isdigit (*++p) &&
					(*++p == 'C' || (isdigit (*p) && *++p == 'C')))
				*b++ = ' ';    /* replace \027&a#C with a space */
			else
				fatal ("unrecognized escape sequence");
		}
		else {
			*b++ = *p;
		}
	}
	*b = '\0';
}

void compact (char *buf) {
	char *b;
	char *p;
	char last = 0;

	for (p = b = buf; *p; p++) {
		if (*p != ' ' || last != ' ')
			last = *b++ = *p;
	}
	*b = '\0';
}

int starts (char *s, char *pat) {
	int n;

	n = strlen (pat);
	if (!strncmp (s, pat, n)) {
		return 1;
	}
	else
		return 0;
}

char *strtrim (char *s) {
	char *p;

	if (!s)
		return NULL;
	else {
		/* find end of string */
		for (p = s; *p; p++)
			;

		/* back up to last character to keep */
		while (p != s && (!*p || isspace (*p))) {
			p--;
		}

		/* write '\0' after it */
		if (!*p || isspace (*p)) {     /* string was ALL whitespace, or empty */
			*p = '\0';
		}
		else {
			*++p = '\0';
		}

		/* find first character to keep */
		for (p = s; isspace (*p); p++)
			;
		return p;
	}
}

char *usage (void) {
	return "usage:  whole_report output_dir [\"supplement date\"] < input_file";
}

void fatal (char *msg) {
	fprintf (stderr, "%s\n", msg);
	exit (1);
}
